@@ -11,7 +11,7 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw(to_ascii to_latin1);
 our %EXPORT_TAGS = ( 'all' => [ @EXPORT ]);
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 our %ANSI_TO_ASCII_TRANSLITERATION = (qw(
          !
@@ -117,37 +117,41 @@ our %ANSI_TO_ASCII_TRANSLITERATION = (qw(
 $ANSI_TO_ASCII_TRANSLITERATION{$_} = '' 
     for (grep {!defined($ANSI_TO_ASCII_TRANSLITERATION{chr $_})} (128..255));
 
-my $non_ascii_char = join("", map {chr} (128..255));
-
 sub to_ascii($) {
     my $text = shift;
     return unless defined $text;
-    $text =~ s/([$non_ascii_char])/$ANSI_TO_ASCII_TRANSLITERATION{$1}/eg;
+    $text =~ s/([\200-\377])/$ANSI_TO_ASCII_TRANSLITERATION{$1}/g;
+                #\octal => \200 = 128, \377 => 255
     return $text;
 }
 
-my %mutation = qw(ae 
-		  Ae 
-		  oe 
-		  Oe 
-		  ue 
-		  Ue );
+my %mutation = qw(
+    ae 
+    Ae 
+    oe 
+    Oe 
+    ue 
+    Ue 
+);
 
 my $vocal = qr/[aeiouAEIOU]/;
 my $consonant = qr/[bcdfghjklmnpqrstvwxzBCDFGHJKLMNPQRSTVWXZ]/;
 my $letter = qr/[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ]/;
 
-my $prefix = qr/(?:[Aa](?:[nb]|u[fs]|bend)|
-                   [Bb]e(?:reit|i||isammen|vor|)|
+my $prefix = qr/(?=[\w]\w)   # to improve speed
+
+                (?:[Aa](?:[nb]|u[fs]|bend)|
+                   [Bb]e(?:reit|i|isammen|vor|)|
                    [Dd](?:a(?>fr|neben|rum|r|)|
                        icke?|
                        rin|
                        urch|
                        rei
                     )|
-                   [Ee]in|
-                   [Ee]nt|
-                   [Ee]r|
+                   [Ee](?:r|
+                          in|
+                          nt|
+                    )|
                    [Ff]e(?:hl|st)|
                    [Ff]rei|
                    (?:[Gg](?:erade|
@@ -215,21 +219,28 @@ sub to_latin1($) {
            (?<! [Mm]eg)
            (?<!  Pent)                # upper case, because of Gruppentter
            (?<! [Ss]of)               # Sofaecke
+           
            ae
+           
+           (?=[\w\.])                     # no  at the end of a word
+           
            (?!rleb)                   # e.g. Ahaerlebnis
            (?!rreg[^i])               #      Malariaerreger
-           (?=[\w])                     # no  at the end of a word
            (?!n\b)                    # even not if in plural
            (?!pid)                    # Choleraepidemie
            (?!in)                     # Kameraeinstellung
            (?!lit)                    # dingsda-elit
            (?!lem)                    # ...element
      	 //gx;
+
         s/(?<=[rtz])ae(?=n\b)//g;                # Eozn, Kapitn, Souvern
+        
         s/phorae/phor/g;             # Epiphor
         s/kenae/ken/g;               # Myken
         s/ovae\b/ov/g;
+        
         s/($town_with_a)r/$1aer/g;
+        
         s/(?<=[mr])(?=ls?\b|li)/ae/g; # Mari
         s/(?<=\b[Pp]r)ae//g;         # Pr...
         s/(?<=bd)ae(?=n)//g;         # Molybdn
@@ -247,7 +258,7 @@ sub to_latin1($) {
           )
          /ae/gx; 
         
-        s/\bAe (?!r[oiu])//gx;         # Ae at beginning of a word, like Aerobic != ra, ren
+        s/\bAe(?!r[oiu])//gx;         # Ae at beginning of a word, like Aerobic != ra, ren
     }
 
 	if (/[Oo]e/) {
@@ -259,20 +270,30 @@ sub to_latin1($) {
 	      (?<! iez)                  # Piezoelektronik
 		  (?<! [Tt]herm)                 # Thermoelektrizitt
           (?<! [Bb]i)                # Bio...
-          (?<!  ktr)                 # Elektro..., 
+          (?<!  [kc]tr)                 # Elektro..., 
           (?<! [Gg]astr)               
           (?<! [Mm]ikr)              # Mikro...
+          (?<! [Ff]err)              # ferro
           (?<! [Rr]homb)
           (?<! [Tt]rapez)
-	      ( [oO] e )
-	      (?! u)
+          (?<! [Hh]ydr)              # Hydro-...
+	      (?<! \b[Cc])               # coeditor or so
+          (?<! \b[Ss]h)                # must be English
+          (?<! c)                    # c doesn't exist in German
+          ( [oO] e )
+	      (?=[\w\.])                     #  not at the end of words
+          (?! [uy])
           (?!ffi[^gn])                     # Koeffizent  != Hffige, Schffin, ...effekt
           (?!ffek)                   # ..effekt
           (?!rot)                    # rot
-          (?!lem)                    # element
+          (?!le[cm])                    # element
           (?!rgo)                    # ergometer
           (?!mpf)                    # empfang
-    	 /$mutation{$1}/egx;
+          (?!ia)                     # typical for latin words e.g. pharmacopoeia
+          (?!last)                   # elastic
+          (?![fv]sk[iy])             # Dostoevsky e.g.
+          (?! c$vocal)               # economic, ...
+         /$mutation{$1}/egx;
          
          s/($town_with_o)r/$1oer/g;
 	 }
@@ -292,11 +313,16 @@ sub to_latin1($) {
     	   )
            (?<![Ss]tat)              # Statue
            (?<!x)                    # Sexuelle
+           (?<![Cc]r)
            ( [uU] e )
            (?= [\w\-])                   # no  at the end of a word
-	       (?! i)                    # Zueilende - -...
+	       (?! [iy])                    # Zueilende - -...
            (?! llst\w)                 # Spirituellste
-         /$mutation{$1}/egx;
+           (?! nce)                    # influence e.g.
+           (?! ntia)
+           (?! s?day)                  # English days
+           (?! some)                   # ...
+          /$mutation{$1}/egx;
          
         s/(?<=[Zz])ue(?=g | n[dfgs] | c[hk] | be[lr] | rn[^t] | ri?ch | bl)//gx; 
         s/(?<=z)ue(?=rnte?[mnrs]?t?\b)//g;
@@ -305,8 +331,8 @@ sub to_latin1($) {
         s/(?<=ga)(?=r(in)?)\b/ue/g;      # ...gauer like Argauer, Thurgauer, ...
          
         {no warnings;
-         s/((?:${prefix}|en)s)?(([tT])n(de?|\b))(?!chen|lein|lich)
-          /$1 ? "$1$2" : "$3uen$4"/xgeo;# Grotuende, but abstnde, Stndchen
+            s/((?:${prefix}|en)s)?(([tT])n(de?|\b))(?!chen|lein|lich)
+             /$1 ? "$1$2" : "$3uen$4"/xgeo;# Grotuende, but abstnde, Stndchen
         }
         s/($prefix s? t)(r(ische?[mnrs]?|
                            i?[ns](nen)?)?\b)/$1ue$2/gx;
@@ -320,7 +346,8 @@ sub to_latin1($) {
         s/ert/euert/g;   # geneuert
         s/re(?=[nv]|s?t)/reue/g;   # reuen
         
-        s/([Au]ssen|
+        s/
+          ([Au]ssen|
            [Dd]oppel|
            [Dd]reh|
            [Ee]ingangs|
@@ -357,7 +384,8 @@ sub to_latin1($) {
       	   (?! isch)                          # Russisch
            (?! land)                          # Ruland
            (?! tau)
-           (?! o)
+           (?! [oy])
+           (?! ia)
           //gx;
           
          # ss =>  with many exceptions
@@ -368,17 +396,17 @@ sub to_latin1($) {
            (?<! su)
            (?<! [bBdDfFhgGHkKlLmMnNpPrRsStTuUvVwWzZ] i )   # 'wissen', -nisse,
            (?<! [dgsklnt] )
-           (?<! [bBdDfFgGhHiIjJkKnNtTwWlLpP] a )     # is a short vocal
+           (?<! [bBdDfFgGhHiIjJkKnNtTwWlLpPvV] a )     # is a short vocal
            (?<! (?<![Ss]t) (?<![fF]) [rR]a)                # Rasse != Strae, fraen
            (?<! [Qq]u a)
            (?<! [bBfFgGhHlLnNpPsSwW] )          # (short vocal) Ablsse, 
-           (?<! [cCdDfFgGhHjJlLmMnNpPrsStTwWzZ] e )           # is very short vocal
+           (?<! [cCdDfFgGhHjJlLmMnNpPrsStTvVwWzZ] e )           # is very short vocal
            (?<! sae)                             # Mensaessen
            (?<! ion )                            # Direktionssekretrin
            (?<! en )                             # dingenssachen 
            (?<! [fFhHoO] l o)
            (?<! (?<![gG]) [rR] o)                # Ross-Schlchter, but Baumgroe          
-           (?<! [bBdDgGkKnNpPzZ] [o])
+           (?<! [bBdDgGkKmMnNpPzZ] [o])
            (?<! [sS]chl )
            (?<! [bBkKuU]e)                       # Kessel
            (?<! [yj])
@@ -399,6 +427,7 @@ sub to_latin1($) {
            (?! te[pu])   # ...steppe, ...steuern
            (?! eins?\b)   # ...-sein
            (?! eit)
+           (?! ett)
            (?! i[vl])    # Massiv, Fossil
            (?! l?ich)  # grsslich  ...sicherung
 	       (?! ge)   # Kreissge
@@ -419,7 +448,8 @@ sub to_latin1($) {
            (?! upp)   # ...-suppe
            (?! anc) # Renaissance
            (?! egn)  # ...-segne
-	      //gxo;
+	       (?! eur)  # Connosseur or so
+          //gxo;
           
           s/(?<= [AaEe]u)                        # drauen
 	        ss 
@@ -487,7 +517,12 @@ sub to_latin1($) {
             s/(?<!)u(?=el|lig)/uss/g;  # Fussel
             s/(?<=[gG]lo)/ss/g;
             s/(?<=sa)(?=in)/ss/g;
-            s/(?<=M[aou])(?=$vocal)/ss/g;  # Massai, Massaker, Massel, Mossul, Musselin
+            s/(?<=\b[Mm][aou])
+              
+              (?!et)         # maet
+              (?=$vocal$consonant|[ae][iy]|eu)
+             /ss/gx;  # Massai, Massaker, Massel, Mossul, Musselin
+            s/Mae/Masse/g;
             s/ma(?=el|ak|ig)/mass/g;
             s/\bma(?=en\w)/mass/g;
             s/((?:\b|$prefix)flo)/$1ss/g;
@@ -516,7 +551,7 @@ sub to_latin1($) {
     s/([aA])pres/$1prs/g;
     s/Apero/Apro/g;
     s/Aragon/Aragn/g;
-    s/deco/dco/g;
+    s/\bdeco\b/dco/g;
     s/socie/soci/g;
     s/([aA])suncion/$1suncin/g;
     s/([aA])ttache/$1ttach/g;
@@ -631,7 +666,7 @@ sub to_latin1($) {
     s/Jerome/Jrme/g;
     s/Kodaly/Kodly/g;
     s/rzitiv/oerzitiv/g;
-    s/nique/niqu/g;
+    #s/nique/niqu/g;
     s/Kalman/Klmn/g;
     s/iberte/ibert/g;
     s/Egalite/galit/g;
@@ -687,7 +722,7 @@ sub to_latin1($) {
 	s/Serac/Srac/g;
 	s/Sevres/Svres/g;
 	s/Stassfurt/Stafurt/g;
-	s/Troms/Troms/g;
+	s/(?<=Troms)(|oe)//g;
 	s/Trouvere/Trouvre/g;
 	s/Tnder/Tnder/g;
 	s/ariete/ariet/g;
@@ -814,11 +849,11 @@ sub to_latin1($) {
     s/groo/grosso/g;
     s/ktl/ktuel/g;
     s/(?<=nn)(?=lle)/ue/g;
-    s/jz\b/juez/g;
+    s/(?<=[gj])(?=z\b)/ue/g;
     s/BDUe/BD/g;
     s/nn\b/nuen/g;
-    s/g(tt|rr)e/gue$1e/g;
-    s/Bl(?=chip|jean|movie)/Blue/g;
+    s/g(?=(tt|rr)[ei])/gue/g;
+    s/(?<=\b[Bb]l)(?=(?!\w)|[bfgjp]|movie|stock|chip)/ue/g; # Blue- [cjb...] is untypical for german, blmen but is
     s/(?<=[Mm]en)ue//g;
     s/Bnos/Buenos/g;
     s/Deng/Dengue/g;
@@ -846,7 +881,7 @@ sub to_latin1($) {
     s/grite/guerite/g;
     s/innd/inuend/g;
     s/(?<=[Gg])(?=st)/ue/g;
-    s/Pblo/Pueblo/g;
+    s/(?<=[Pp])(?=blo)/ue/g;
     s/(?<=[Pp])(?=rto)/ue/g;
     s/Re(?=[nv])/Reue/g;
     s/Saml/Samuel/g;
@@ -857,11 +892,42 @@ sub to_latin1($) {
     s/cker/Uecker/g;
     s/sll/suell/g;
     s/nnll/nnuell/g;
-
+    s/(?<!\w)(?=ron)/ae/g;       # aeronautic e.g.
     s/\beinzng/einzueng/g;
     s/\bt(?=s?t\b)/tue/g;
     s/Zei/Zeiss/g;          # I'm coming from Jena, Zeiss' hometown and that
                              # really looks better :-)
+    s/(?<=airma)/ss/g;
+    s/Dda/Daeda/g;
+    s/Bna/Buena/g;
+    s/(?<=can)/oe/g;        # english for "Kanu"
+    s/(canv|carc)a/$1ass/g;      # english for "prfend", ...
+    s/Caandra/Cassandra/g;
+    s/Citr(oe|)n/Citron/g; 
+    s/\bdmon\b/daemon/g;        # lowercase is wrong German (but not for "dmonenhaft"), thus it must be the english version
+    s/([A-Z]*)(Ae|Oe|Ue)\b/$1$mutation{$2}/g;   # must be an abbreviation ending on ,,   
+    s/jo/joss/g;
+    s/Suex/Sussex/g;
+    s/(?<![Oo])([Bb]|rh)oe\b/$1/g;   # ...-rrh, Windb, != Oboe
+    s/Malmoe/Malm/g;
+    s/(fl|tr?)nt\b/$1uent/g;
+    s/gse/guese/g;
+    s/(?<=[sh])(?=ver)/oe/g;       # whomsever, whoever
+    s/potats/potatoes/g;
+    s/(?<=\b[fd])(?=[lt]\b)/ue/g;  # fuel, duel, duet, ...
+    s/Dreyfu/Dreyfuss/g;
+    s/(?<=[Gg])(?=lic)/ae/g;
+    s/(?<=[Mm])(?=stro)/ae/g;
+    s/(?<=\b[dg])(?=s(\b|n))/oe/g;  # does, goes
+    s/mint\b/minuet/g;
+    s/(?<=trespa)/ss/g;
+    s/N(oe|)l\b/Nol/g;
+    s/\bgss\b/guess/g;
+    s/mutl/mutuel/g;
+    s/il/oeil/g;
+    s/mzzi/muezzi/g;
+    s/psy/poesy/g;
+    s/(?<=[Tt]omat)/oe/g;
     return $_;
 }
 
@@ -963,12 +1029,20 @@ but the price is that in some rare cases of words with multiple umlauts
 (like "Hkeltlle") some buggy conversions can occur.
 Please tell me if you find such words.
 
+The C<to_latin1> method also has some knowledge to work with some basic English.
+(So that some words don't confuse everything and you can
+also use some code snippets in your text).
+However, it is very recommended to use American English instead of British English.
+Espeically many plural forms (ending on "oes") are hard to handle,
+and often I decided not to implement an extra rule as
+it is a C<Lingua::DE::*> module and not an English one.
+
 =head1 TESTS
 
 The test scripts (called by e.g. C<make test>) need a long time.
 The reason is that I test it with a huge german word list. Normally you can skip
 this test if there is no failing in the first few seconds. However, the tests
-also have a progress bar, so that you can see the advances :-)
+also have a progress bar (either a Term::ProgressBar if installed or just a simple text output), so that you can see the advances :-)
 
 There are two major reasons why I added so many words to test even to the CPAN
 release. On the one hand, I wanted to give you a chance to detect strange
@@ -980,6 +1054,10 @@ a chance to detect yourself whether a C<to_latin1> result is a bug or a feature.
 (Just search through the content of the test files to determine whether a
 strange looking word is tested for and thus wanted).
 
+There is also a test with common 1000 English words (having an ae,oe,ue or ss inside),
+as German is nowadays often mixed with a lot of them,
+and this module should not be confused with them.
+
 =head1 AUTHOR
 
 Janek Schleicher, E<lt>bigj@kamelfreund.deE<gt>
